@@ -1,5 +1,6 @@
 import os
 import sys
+import sysconfig
 import warnings
 import ctypes
 import ctypes.util
@@ -10,11 +11,16 @@ from galpy.orbit_src.integrateFullOrbit import _parse_pot
 #Find and load the library
 _lib= None
 outerr= None
+PY3= sys.version > '3'
+if PY3: #pragma: no cover
+    _ext_suffix= sysconfig.get_config_var('EXT_SUFFIX')
+else:
+    _ext_suffix= '.so'
 for path in sys.path:
     try:
-        _lib = ctypes.CDLL(os.path.join(path,'galpy_actionAngle_c.so'))
-    except OSError, e:
-        if os.path.exists(os.path.join(path,'galpy_actionAngle_c.so')): #pragma: no cover
+        _lib = ctypes.CDLL(os.path.join(path,'galpy_actionAngle_c%s' % _ext_suffix))
+    except OSError as e:
+        if os.path.exists(os.path.join(path,'galpy_actionAngle_c%s' % _ext_suffix)): #pragma: no cover
             outerr= e
         _lib = None
     else:
@@ -24,7 +30,7 @@ if _lib is None: #pragma: no cover
         warnings.warn("actionAngleAdiabatic_c extension module not loaded, because of error '%s' " % outerr,
                       galpyWarning)
     else:
-        warnings.warn("actionAngleAdiabatic_c extension module not loaded, because galpy_actionAngle_c.so image was not found",
+        warnings.warn("actionAngleAdiabatic_c extension module not loaded, because galpy_actionAngle_c%s image was not found" % _ext_suffix,
                       galpyWarning)
     _ext_loaded= False
 else:
