@@ -3,6 +3,10 @@
 */
 #ifndef __GALPY_ACTIONANGLE_H__
 #define __GALPY_ACTIONANGLE_H__
+#ifdef __cplusplus
+extern "C" {
+#endif
+#include <stdbool.h>
 #include <gsl/gsl_roots.h>
 #include <gsl/gsl_spline.h>
 #include "interp_2d.h"
@@ -35,6 +39,8 @@ struct pragmasolver{
 /*
   Function declarations
 */
-double evaluatePotentials(double,double,int, struct potentialArg *);
-void parse_actionAngleArgs(int,struct potentialArg *,int *,double *);
+  void parse_actionAngleArgs(int,struct potentialArg *,int *,double *,bool);
+#ifdef __cplusplus
+}
+#endif
 #endif /* actionAngle.h */
