@@ -1,7 +1,11 @@
 #ifndef __INTEGRATEFULLORBIT_H__
 #define __INTEGRATEFULLORBIT_H__
+#ifdef __cplusplus
+extern "C" {
+#endif
 #include <galpy_potentials.h>
 void parse_leapFuncArgs_Full(int, struct potentialArg *,int *,double *);
-double calcRforce(double,double,double,double,int,struct potentialArg *);
-double calczforce(double,double,double,double,int,struct potentialArg *);
+#ifdef __cplusplus
+}
+#endif
 #endif /* integrateFullOrbit.h */
