@@ -5,20 +5,111 @@ import pickle
 import numpy as nu
 from scipy import integrate
 import galpy.util.bovy_plot as plot
+from galpy.util import config
+from galpy.util.bovy_conversion import physical_conversion,\
+    potential_physical_input, freq_in_Gyr
 from galpy.potential_src.Potential import Potential, PotentialError, lindbladR
 from galpy.potential_src.plotRotcurve import plotRotcurve
 from galpy.potential_src.plotEscapecurve import _INF, plotEscapecurve
+_APY_LOADED= True
+try:
+    from astropy import units
+except ImportError:
+    _APY_LOADED= False
 class planarPotential(object):
     """Class representing 2D (R,\phi) potentials"""
-    def __init__(self,amp=1.):
+    def __init__(self,amp=1.,ro=None,vo=None):
         self._amp= 1.
         self.dim= 2
         self.isNonAxi= True #Gets reset by planarAxiPotential
         self.isRZ= False
         self.hasC= False
         self.hasC_dxdv= False
+        # Parse ro and vo
+        if ro is None:
+            self._ro= config.__config__.getfloat('normalization','ro')
+            self._roSet= False
+        else:
+            if _APY_LOADED and isinstance(ro,units.Quantity):
+                ro= ro.to(units.kpc).value
+            self._ro= ro
+            self._roSet= True
+        if vo is None:
+            self._vo= config.__config__.getfloat('normalization','vo')
+            self._voSet= False
+        else:
+            if _APY_LOADED and isinstance(vo,units.Quantity):
+                vo= vo.to(units.km/units.s).value
+            self._vo= vo
+            self._voSet= True
         return None
 
+    def turn_physical_off(self):
+        """
+        NAME:
+
+           turn_physical_off
+
+        PURPOSE:
+
+           turn off automatic returning of outputs in physical units
+
+        INPUT:
+
+           (none)
+
+        OUTPUT:
+
+           (none)
+
+        HISTORY:
+
+           2016-01-30 - Written - Bovy (UofT)
+
+        """
+        self._roSet= False
+        self._voSet= False
+        return None
+
+    def turn_physical_on(self,ro=None,vo=None):
+        """
+        NAME:
+
+           turn_physical_on
+
+        PURPOSE:
+
+           turn on automatic returning of outputs in physical units
+
+        INPUT:
+
+           ro= reference distance (kpc; can be Quantity)
+
+           vo= reference velocity (km/s; can be Quantity)
+
+        OUTPUT:
+
+           (none)
+
+        HISTORY:
+
+           2016-01-30 - Written - Bovy (UofT)
+
+        """
+        self._roSet= True
+        self._voSet= True
+        if not ro is None:
+            if _APY_LOADED and isinstance(ro,units.Quantity):
+                ro= ro.to(units.kpc).value
+            self._ro= ro
+        if not vo is None:
+            if _APY_LOADED and isinstance(vo,units.Quantity):
+                vo= vo.to(units.km/units.s).value
+            self._vo= vo
+        return None
+
+    @potential_physical_input
+    @physical_conversion('energy',pop=True)
     def __call__(self,R,phi=0.,t=0.,dR=0,dphi=0):
         """
         NAME:
@@ -31,13 +122,11 @@ class planarPotential(object):
 
         INPUT: 
 
-           R - Cylindrica radius
+           R - Cylindrica radius (can be Quantity)
 
-           phi= azimuth (optional)
+           phi= azimuth (optional; can be Quantity)
 
-           t= time (optional)
-
-           dR=, dphi= if set to non-zero integers, return the dR,dphi't derivative
+           t= time (optional; can be Quantity)
 
         OUTPUT:
 
@@ -48,22 +137,28 @@ class planarPotential(object):
            2010-07-13 - Written - Bovy (NYU)
 
         """
+        return self._call_nodecorator(R,phi=phi,t=t,dR=dR,dphi=dphi)
+
+    def _call_nodecorator(self,R,phi=0.,t=0.,dR=0,dphi=0):
+        # Separate, so it can be used during orbit integration
         if dR == 0 and dphi == 0:
             try:
                 return self._amp*self._evaluate(R,phi=phi,t=t)
             except AttributeError: #pragma: no cover
                 raise PotentialError("'_evaluate' function not implemented for this potential")
         elif dR == 1 and dphi == 0:
-            return -self.Rforce(R,phi=phi,t=t)
+            return -self.Rforce(R,phi=phi,t=t,use_physical=False)
         elif dR == 0 and dphi == 1:
-            return -self.phiforce(R,phi=phi,t=t)
+            return -self.phiforce(R,phi=phi,t=t,use_physical=False)
         elif dR == 2 and dphi == 0:
-            return self.R2deriv(R,phi=phi,t=t)
+            return self.R2deriv(R,phi=phi,t=t,use_physical=False)
         elif dR == 0 and dphi == 2:
-            return self.phi2deriv(R,phi=phi,t=t)
+            return self.phi2deriv(R,phi=phi,t=t,use_physical=False)
         elif dR == 1 and dphi == 1:
-            return self.Rphideriv(R,phi=phi,t=t)
+            return self.Rphideriv(R,phi=phi,t=t,use_physical=False)
 
+    @potential_physical_input
+    @physical_conversion('force',pop=True)
     def Rforce(self,R,phi=0.,t=0.):
         """
         NAME:
@@ -76,11 +171,11 @@ class planarPotential(object):
 
         INPUT:
 
-           R - Cylindrical radius
+           R - Cylindrical radius (can be Quantity)
 
-           phi= azimuth (optional)
+           phi= azimuth (optional; can be Quantity)
 
-           t= time (optional)
+           t= time (optional; can be Quantity)
 
         OUTPUT:
 
@@ -91,11 +186,17 @@ class planarPotential(object):
            2010-07-13 - Written - Bovy (NYU)
 
         """
+        return self._Rforce_nodecorator(R,phi=phi,t=t)
+
+    def _Rforce_nodecorator(self,R,phi=0.,t=0.):
+        # Separate, so it can be used during orbit integration
         try:
             return self._amp*self._Rforce(R,phi=phi,t=t)
         except AttributeError: #pragma: no cover
             raise PotentialError("'_Rforce' function not implemented for this potential")
 
+    @potential_physical_input
+    @physical_conversion('force',pop=True)
     def phiforce(self,R,phi=0.,t=0.):
         """
         NAME:
@@ -108,26 +209,32 @@ class planarPotential(object):
 
         INPUT:
 
-           R - Cylindrical radius
+           R - Cylindrical radius (can be Quantity)
 
-           phi= azimuth (optional)
+           phi= azimuth (optional; can be Quantity)
 
-           t= time (optional)
+           t= time (optional; can be Quantity)
 
         OUTPUT:
 
-           F_\phi(R,(\phi,t)))
+           F_phi(R,(phi,t)))
 
         HISTORY:
 
            2010-07-13 - Written - Bovy (NYU)
 
         """
+        return self._phiforce_nodecorator(R,phi=phi,t=t)
+       
+    def _phiforce_nodecorator(self,R,phi=0.,t=0.):
+        # Separate, so it can be used during orbit integration
         try:
             return self._amp*self._phiforce(R,phi=phi,t=t)
         except AttributeError: #pragma: no cover
             raise PotentialError("'_phiforce' function not implemented for this potential")
 
+    @potential_physical_input
+    @physical_conversion('forcederivative',pop=True)
     def R2deriv(self,R,phi=0.,t=0.):
         """
         NAME:
@@ -140,11 +247,11 @@ class planarPotential(object):
 
         INPUT:
 
-           R - Cylindrical radius
+           R - Cylindrical radius (can be Quantity)
 
-           phi= azimuth (optional)
+           phi= azimuth (optional; can be Quantity)
 
-           t= time (optional)
+           t= time (optional; can be Quantity)
 
         OUTPUT:
 
@@ -160,6 +267,8 @@ class planarPotential(object):
         except AttributeError: #pragma: no cover
             raise PotentialError("'_R2deriv' function not implemented for this potential")      
 
+    @potential_physical_input
+    @physical_conversion('forcederivative',pop=True)
     def phi2deriv(self,R,phi=0.,t=0.):
         """
         NAME:
@@ -172,11 +281,11 @@ class planarPotential(object):
 
         INPUT:
 
-           R - Cylindrical radius
+           R - Cylindrical radius (can be Quantity)
 
-           phi= azimuth (optional)
+           phi= azimuth (optional; can be Quantity)
 
-           t= time (optional)
+           t= time (optional; can be Quantity)
 
         OUTPUT:
 
@@ -192,6 +301,8 @@ class planarPotential(object):
         except AttributeError: #pragma: no cover
             raise PotentialError("'_phi2deriv' function not implemented for this potential")      
 
+    @potential_physical_input
+    @physical_conversion('forcederivative',pop=True)
     def Rphideriv(self,R,phi=0.,t=0.):
         """
         NAME:
@@ -204,11 +315,11 @@ class planarPotential(object):
 
         INPUT:
 
-           R - Cylindrical radius
+           R - Cylindrical radius (can be Quantity)
 
-           phi= azimuth (optional)
+           phi= azimuth (optional can be Quantity)
 
-           t= time (optional)
+           t= time (optional; can be Quantity)
 
         OUTPUT:
 
@@ -231,7 +342,7 @@ class planarPotential(object):
         PURPOSE:
            plot the potential
         INPUT:
-           Rrange - range
+           Rrange - range (can be Quantity)
            grid - number of points to plot
            savefilename - save to or restore from this savefile (pickle)
            +bovy_plot(*args,**kwargs)
@@ -244,8 +355,8 @@ class planarPotential(object):
 
 class planarAxiPotential(planarPotential):
     """Class representing axisymmetric planar potentials"""
-    def __init__(self,amp=1.):
-        planarPotential.__init__(self,amp=amp)
+    def __init__(self,amp=1.,ro=None,vo=None):
+        planarPotential.__init__(self,amp=amp,ro=ro,vo=vo)
         self.isNonAxi= False
         return None
     
@@ -288,7 +399,9 @@ class planarAxiPotential(planarPotential):
         """
         return 0.
 
-    def vcirc(self,R):
+    @potential_physical_input
+    @physical_conversion('velocity',pop=True)
+    def vcirc(self,R,phi=None):
         """
         
         NAME:
@@ -303,8 +416,10 @@ class planarAxiPotential(planarPotential):
         
             Pot - Potential instance or list of such instances
         
-            R - Galactocentric radius
+            R - Galactocentric radius (can be Quantity)
         
+            phi= (None) azimuth to use for non-axisymmetric potentials
+
         OUTPUT:
         
             circular rotation velocity
@@ -313,9 +428,13 @@ class planarAxiPotential(planarPotential):
         
             2011-10-09 - Written - Bovy (IAS)
         
-        """
-        return nu.sqrt(R*-self.Rforce(R))       
+            2016-06-15 - Added phi= keyword for non-axisymmetric potential - Bovy (UofT)
 
+        """
+        return nu.sqrt(R*-self.Rforce(R,phi=phi,use_physical=False))
+
+    @potential_physical_input
+    @physical_conversion('frequency',pop=True)
     def omegac(self,R):
         """
         
@@ -331,7 +450,7 @@ class planarAxiPotential(planarPotential):
         
             Pot - Potential instance or list of such instances
         
-            R - Galactocentric radius
+            R - Galactocentric radius (can be Quantity)
         
         OUTPUT:
         
@@ -342,8 +461,10 @@ class planarAxiPotential(planarPotential):
             2011-10-09 - Written - Bovy (IAS)
         
         """
-        return nu.sqrt(-self.Rforce(R)/R)       
+        return nu.sqrt(-self.Rforce(R,use_physical=False)/R)       
 
+    @potential_physical_input
+    @physical_conversion('frequency',pop=True)
     def epifreq(self,R):
         """
         
@@ -357,7 +478,7 @@ class planarAxiPotential(planarPotential):
         
         INPUT:
         
-           R - Galactocentric radius
+           R - Galactocentric radius (can be Quantity)
         
         OUTPUT:
         
@@ -368,8 +489,10 @@ class planarAxiPotential(planarPotential):
            2011-10-09 - Written - Bovy (IAS)
         
         """
-        return nu.sqrt(self.R2deriv(R)-3./R*self.Rforce(R))
+        return nu.sqrt(self.R2deriv(R,use_physical=False)
+                       -3./R*self.Rforce(R,use_physical=False))
 
+    @physical_conversion('position',pop=True)
     def lindbladR(self,OmegaP,m=2,**kwargs):
         """
         
@@ -383,7 +506,7 @@ class planarAxiPotential(planarPotential):
         
         INPUT:
         
-           OmegaP - pattern speed
+           OmegaP - pattern speed (can be Quantity)
 
            m= order of the resonance (as in m(O-Op)=kappa (negative m for outer)
               use m='corotation' for corotation
@@ -398,8 +521,12 @@ class planarAxiPotential(planarPotential):
            2011-10-09 - Written - Bovy (IAS)
         
         """
-        return lindbladR(self,OmegaP,m=m,**kwargs)
+        if _APY_LOADED and isinstance(OmegaP,units.Quantity):
+            OmegaP= OmegaP.to(1/units.Gyr).value/freq_in_Gyr(self._vo,self._ro)
+        return lindbladR(self,OmegaP,m=m,use_physical=False,**kwargs)
 
+    @potential_physical_input
+    @physical_conversion('velocity',pop=True)
     def vesc(self,R):
         """
 
@@ -415,7 +542,7 @@ class planarAxiPotential(planarPotential):
 
             Pot - Potential instances or list thereof
 
-            R - Galactocentric radius
+            R - Galactocentric radius (can be Quantity)
 
         OUTPUT:
 
@@ -426,7 +553,8 @@ class planarAxiPotential(planarPotential):
             2011-10-09 - Written - Bovy (IAS)
 
         """
-        return nu.sqrt(2.*(self(_INF)-self(R)))
+        return nu.sqrt(2.*(self(_INF,use_physical=False)
+                           -self(R,use_physical=False)))
         
     def plotRotcurve(self,*args,**kwargs):
         """
@@ -440,7 +568,7 @@ class planarAxiPotential(planarPotential):
 
         INPUT:
 
-           Rrange - range
+           Rrange - range (can be Quantity)
 
            grid - number of points to plot
 
@@ -471,7 +599,7 @@ class planarAxiPotential(planarPotential):
 
         INPUT:
 
-           Rrange - range
+           Rrange - range (can be Quantity)
 
            grid - number of points to plot
 
@@ -506,8 +634,11 @@ class planarPotentialFromRZPotential(planarAxiPotential):
         HISTORY:
            2010-07-13 - Written - Bovy (NYU)
         """
-        planarAxiPotential.__init__(self,amp=1.)
-        self._RZPot= RZPot
+        planarAxiPotential.__init__(self,amp=1.,ro=RZPot._ro,vo=RZPot._vo)
+        # Also transfer roSet and voSet
+        self._roSet= RZPot._roSet
+        self._voSet= RZPot._voSet
+        self._Pot= RZPot
         self.hasC= RZPot.hasC
         self.hasC_dxdv= RZPot.hasC_dxdv
         return None
@@ -527,7 +658,7 @@ class planarPotentialFromRZPotential(planarAxiPotential):
         HISTORY:
            2010-07-13 - Written - Bovy (NYU)
         """
-        return self._RZPot(R,0.,t=t)
+        return self._Pot(R,0.,t=t,use_physical=False)
             
     def _Rforce(self,R,phi=0.,t=0.):
         """
@@ -544,7 +675,7 @@ class planarPotentialFromRZPotential(planarAxiPotential):
         HISTORY:
            2010-07-13 - Written - Bovy (NYU)
         """
-        return self._RZPot.Rforce(R,0.,t=t)
+        return self._Pot.Rforce(R,0.,t=t,use_physical=False)
 
     def _R2deriv(self,R,phi=0.,t=0.):
         """
@@ -561,7 +692,7 @@ class planarPotentialFromRZPotential(planarAxiPotential):
         HISTORY:
            2011-10-09 - Written - Bovy (IAS)
         """
-        return self._RZPot.R2deriv(R,0.,t=t)
+        return self._Pot.R2deriv(R,0.,t=t,use_physical=False)
             
 def RZToplanarPotential(RZPot):
     """
@@ -601,7 +732,193 @@ def RZToplanarPotential(RZPot):
     else:
         raise PotentialError("Input to 'RZToplanarPotential' is neither an RZPotential-instance or a list of such instances")
 
-def evaluateplanarPotentials(R,Pot,phi=None,t=0.,dR=0,dphi=0):
+class planarPotentialFromFullPotential(planarPotential):
+    """Class that represents a planar potential derived from a non-axisymmetric
+    3D potential"""
+    def __init__(self,Pot):
+        """
+        NAME:
+           __init__
+        PURPOSE:
+           Initialize
+        INPUT:
+           Pot - Potential instance
+        OUTPUT:
+           planarPotential instance
+        HISTORY:
+           2016-06-02 - Written - Bovy (UofT)
+        """
+        planarPotential.__init__(self,amp=1.,ro=Pot._ro,vo=Pot._vo)
+        # Also transfer roSet and voSet
+        self._roSet= Pot._roSet
+        self._voSet= Pot._voSet
+        self._Pot= Pot
+        self.hasC= Pot.hasC
+        self.hasC_dxdv= Pot.hasC_dxdv
+        return None
+
+    def _evaluate(self,R,phi=0.,t=0.):
+        """
+        NAME:
+           _evaluate
+        PURPOSE:
+           evaluate the potential
+        INPUT:
+           R
+           phi
+           t
+        OUTPUT:
+          Pot(R(,\phi,t))
+        HISTORY:
+           2016-06-02 - Written - Bovy (UofT)
+        """
+        return self._Pot(R,0.,phi=phi,t=t,use_physical=False)
+            
+    def _Rforce(self,R,phi=0.,t=0.):
+        """
+        NAME:
+           _Rforce
+        PURPOSE:
+           evaluate the radial force
+        INPUT:
+           R
+           phi
+           t
+        OUTPUT:
+          F_R(R(,\phi,t))
+        HISTORY:
+           2016-06-02 - Written - Bovy (UofT)
+        """
+        return self._Pot.Rforce(R,0.,phi=phi,t=t,use_physical=False)
+
+    def _phiforce(self,R,phi=0.,t=0.):
+        """
+        NAME:
+           _phiforce
+        PURPOSE:
+           evaluate the azimuthal force
+        INPUT:
+           R
+           phi
+           t
+        OUTPUT:
+          F_phi(R(,\phi,t))
+        HISTORY:
+           2016-06-02 - Written - Bovy (UofT)
+        """
+        return self._Pot.phiforce(R,0.,phi=phi,t=t,use_physical=False)
+
+    def _R2deriv(self,R,phi=0.,t=0.):
+        """
+        NAME:
+           _R2deriv
+        PURPOSE:
+           evaluate the second radial derivative
+        INPUT:
+           R
+           phi
+           t
+        OUTPUT:
+           d2phi/dR2
+        HISTORY:
+           2016-06-02 - Written - Bovy (UofT)
+        """
+        return self._Pot.R2deriv(R,0.,phi=phi,t=t,use_physical=False)
+            
+    def _phi2deriv(self,R,phi=0.,t=0.):
+        """
+        NAME:
+           _phi2deriv
+        PURPOSE:
+           evaluate the second azimuthal derivative
+        INPUT:
+           R
+           phi
+           t
+        OUTPUT:
+           d2phi/dphi2
+        HISTORY:
+           2016-06-02 - Written - Bovy (UofT)
+        """
+        return self._Pot.phi2deriv(R,0.,phi=phi,t=t,use_physical=False)
+            
+    def _Rphideriv(self,R,phi=0.,t=0.):
+        """
+        NAME:
+           _Rphideriv
+        PURPOSE:
+           evaluate the mixed radial-azimuthal derivative
+        INPUT:
+           R
+           phi
+           t
+        OUTPUT:
+           d2phi/dRdphi
+        HISTORY:
+           2016-06-02 - Written - Bovy (UofT)
+        """
+        return self._Pot.Rphideriv(R,0.,phi=phi,t=t,use_physical=False)
+            
+    def OmegaP(self):
+        """
+        NAME:
+           OmegaP
+        PURPOSE:
+           return the pattern speed
+        INPUT:
+           (none)
+        OUTPUT:
+           pattern speed
+        HISTORY:
+           2016-05-31 - Written - Bovy (UofT)
+        """
+        return self._Pot.OmegaP()
+            
+def toPlanarPotential(Pot):
+    """
+    NAME:
+
+       toPlanarPotential
+
+    PURPOSE:
+
+       convert an Potential to a planarPotential in the mid-plane (z=0)
+
+    INPUT:
+
+       Pot - Potential instance or list of such instances (existing planarPotential instances are just copied to the output)
+
+    OUTPUT:
+
+       planarPotential instance(s)
+
+    HISTORY:
+
+       2016-06-11 - Written - Bovy (UofT)
+
+    """
+    if isinstance(Pot,list):
+        out= []
+        for pot in Pot:
+            if isinstance(pot,planarPotential):
+                out.append(pot)
+            elif pot.isNonAxi:
+                out.append(planarPotentialFromFullPotential(pot))
+            else:
+                out.append(planarPotentialFromRZPotential(pot))
+        return out
+    elif isinstance(Pot,Potential) and Pot.isNonAxi:
+        return planarPotentialFromFullPotential(Pot)
+    elif isinstance(Pot,Potential):
+        return planarPotentialFromRZPotential(Pot)
+    elif isinstance(Pot,planarPotential):
+        return Pot
+    else:
+        raise PotentialError("Input to 'toPlanarPotential' is neither an Potential-instance or a list of such instances")
+
+@potential_physical_input
+@physical_conversion('energy',pop=True)
+def evaluateplanarPotentials(Pot,R,phi=None,t=0.,dR=0,dphi=0):
     """
     NAME:
 
@@ -613,13 +930,13 @@ def evaluateplanarPotentials(R,Pot,phi=None,t=0.,dR=0,dphi=0):
 
     INPUT:
 
-       R - Cylindrical radius
-
        Pot - (list of) planarPotential instance(s)
 
-       phi= azimuth (optional)
+       R - Cylindrical radius (can be Quantity)
 
-       t= time (optional)
+       phi= azimuth (optional; can be Quantity)
+
+       t= time (optional; can be Quantity)
 
        dR=, dphi= if set to non-zero integers, return the dR,dphi't derivative instead
 
@@ -632,32 +949,33 @@ def evaluateplanarPotentials(R,Pot,phi=None,t=0.,dR=0,dphi=0):
        2010-07-13 - Written - Bovy (NYU)
 
     """
+    return _evaluateplanarPotentials(Pot,R,phi=phi,t=t,dR=dR,dphi=dphi)
+
+def _evaluateplanarPotentials(Pot,R,phi=None,t=0.,dR=0,dphi=0):
+    from galpy.potential_src.Potential import _isNonAxi
     isList= isinstance(Pot,list)
-    if isList:
-        isAxis= [not p.isNonAxi for p in Pot]
-        nonAxi= not nu.prod(nu.array(isAxis))
-    else:
-        nonAxi= Pot.isNonAxi
+    nonAxi= _isNonAxi(Pot)
     if nonAxi and phi is None:
         raise PotentialError("The (list of) planarPotential instances is non-axisymmetric, but you did not provide phi")
-    if isinstance(Pot,list) \
-            and nu.all([isinstance(p,planarPotential) for p in Pot]):
+    if isList and nu.all([isinstance(p,planarPotential) for p in Pot]):
         sum= 0.
         for pot in Pot:
             if nonAxi:
-                sum+= pot(R,phi=phi,t=t,dR=dR,dphi=dphi)
+                sum+= pot._call_nodecorator(R,phi=phi,t=t,dR=dR,dphi=dphi)
             else:
-                sum+= pot(R,t=t,dR=dR,dphi=dphi)
+                sum+= pot._call_nodecorator(R,t=t,dR=dR,dphi=dphi)
         return sum
     elif isinstance(Pot,planarPotential):
         if nonAxi:
-            return Pot(R,phi=phi,t=t,dR=dR,dphi=dphi)
+            return Pot._call_nodecorator(R,phi=phi,t=t,dR=dR,dphi=dphi)
         else:
-            return Pot(R,t=t,dR=dR,dphi=dphi)
+            return Pot._call_nodecorator(R,t=t,dR=dR,dphi=dphi)
     else: #pragma: no cover 
         raise PotentialError("Input to 'evaluatePotentials' is neither a Potential-instance or a list of such instances")
 
-def evaluateplanarRforces(R,Pot,phi=None,t=0.):
+@potential_physical_input
+@physical_conversion('force',pop=True)
+def evaluateplanarRforces(Pot,R,phi=None,t=0.):
     """
     NAME:
 
@@ -669,13 +987,13 @@ def evaluateplanarRforces(R,Pot,phi=None,t=0.):
 
     INPUT:
 
-       R - Cylindrical radius
-
        Pot - (list of) planarPotential instance(s)
 
-       phi= azimuth (optional)
+       R - Cylindrical radius (can be Quantity)
 
-       t= time (optional)
+       phi= azimuth (optional can be Quantity)
+
+       t= time (optional; can be Quantity)
 
     OUTPUT:
 
@@ -686,12 +1004,13 @@ def evaluateplanarRforces(R,Pot,phi=None,t=0.):
        2010-07-13 - Written - Bovy (NYU)
 
     """
+    return _evaluateplanarRforces(Pot,R,phi=phi,t=t)
+
+def _evaluateplanarRforces(Pot,R,phi=None,t=0.):
+    """Raw, undecorated function for internal use"""
+    from galpy.potential_src.Potential import _isNonAxi
     isList= isinstance(Pot,list)
-    if isList:
-        isAxis= [not p.isNonAxi for p in Pot]
-        nonAxi= not nu.prod(nu.array(isAxis))
-    else:
-        nonAxi= Pot.isNonAxi
+    nonAxi= _isNonAxi(Pot)
     if nonAxi and phi is None:
         raise PotentialError("The (list of) planarPotential instances is non-axisymmetric, but you did not provide phi")
     if isinstance(Pot,list) \
@@ -699,19 +1018,21 @@ def evaluateplanarRforces(R,Pot,phi=None,t=0.):
         sum= 0.
         for pot in Pot:
             if nonAxi:
-                sum+= pot.Rforce(R,phi=phi,t=t)
+                sum+= pot._Rforce_nodecorator(R,phi=phi,t=t)
             else:
-                sum+= pot.Rforce(R,t=t)
+                sum+= pot._Rforce_nodecorator(R,t=t)
         return sum
     elif isinstance(Pot,planarPotential):
         if nonAxi:
-            return Pot.Rforce(R,phi=phi,t=t)
+            return Pot._Rforce_nodecorator(R,phi=phi,t=t)
         else:
-            return Pot.Rforce(R,t=t)
+            return Pot._Rforce_nodecorator(R,t=t)
     else: #pragma: no cover 
         raise PotentialError("Input to 'evaluatePotentials' is neither a Potential-instance or a list of such instances")
 
-def evaluateplanarphiforces(R,Pot,phi=None,t=0.):
+@potential_physical_input
+@physical_conversion('force',pop=True)
+def evaluateplanarphiforces(Pot,R,phi=None,t=0.):
     """
     NAME:
 
@@ -723,13 +1044,13 @@ def evaluateplanarphiforces(R,Pot,phi=None,t=0.):
 
     INPUT:
 
-       R - Cylindrical radius
-
        Pot - (list of) planarPotential instance(s)
 
-       phi= azimuth (optional)
+       R - Cylindrical radius (can be Quantity)
 
-       t= time (optional)
+       phi= azimuth (optional; can be Quantity)
+
+       t= time (optional; can be Quantity)
 
     OUTPUT:
 
@@ -740,12 +1061,12 @@ def evaluateplanarphiforces(R,Pot,phi=None,t=0.):
        2010-07-13 - Written - Bovy (NYU)
 
     """
+    return _evaluateplanarphiforces(Pot,R,phi=phi,t=t)
+
+def _evaluateplanarphiforces(Pot,R,phi=None,t=0.):
+    from galpy.potential_src.Potential import _isNonAxi
     isList= isinstance(Pot,list)
-    if isList:
-        isAxis= [not p.isNonAxi for p in Pot]
-        nonAxi= not nu.prod(nu.array(isAxis))
-    else:
-        nonAxi= Pot.isNonAxi
+    nonAxi= _isNonAxi(Pot)
     if nonAxi and phi is None:
         raise PotentialError("The (list of) planarPotential instances is non-axisymmetric, but you did not provide phi")
     if isinstance(Pot,list) \
@@ -753,19 +1074,21 @@ def evaluateplanarphiforces(R,Pot,phi=None,t=0.):
         sum= 0.
         for pot in Pot:
             if nonAxi:
-                sum+= pot.phiforce(R,phi=phi,t=t)
+                sum+= pot._phiforce_nodecorator(R,phi=phi,t=t)
             else:
-                sum+= pot.phiforce(R,t=t)
+                sum+= pot._phiforce_nodecorator(R,t=t)
         return sum
     elif isinstance(Pot,planarPotential):
         if nonAxi:
-            return Pot.phiforce(R,phi=phi,t=t)
+            return Pot._phiforce_nodecorator(R,phi=phi,t=t)
         else:
-            return Pot.phiforce(R,t=t)
+            return Pot._phiforce_nodecorator(R,t=t)
     else: #pragma: no cover 
         raise PotentialError("Input to 'evaluatePotentials' is neither a Potential-instance or a list of such instances")
 
-def evaluateplanarR2derivs(R,Pot,phi=None,t=0.):
+@potential_physical_input
+@physical_conversion('forcederivative',pop=True)
+def evaluateplanarR2derivs(Pot,R,phi=None,t=0.):
     """
     NAME:
 
@@ -777,13 +1100,13 @@ def evaluateplanarR2derivs(R,Pot,phi=None,t=0.):
 
     INPUT:
 
-       R - Cylindrical radius
-
        Pot - (list of) planarPotential instance(s)
 
-       phi= azimuth (optional)
+       R - Cylindrical radius (can be Quantity)
 
-       t= time (optional)
+       phi= azimuth (optional; can be Quantity)
+
+       t= time (optional; can be Quantity)
 
     OUTPUT:
 
@@ -794,12 +1117,9 @@ def evaluateplanarR2derivs(R,Pot,phi=None,t=0.):
        2010-10-09 - Written - Bovy (IAS)
 
     """
+    from galpy.potential_src.Potential import _isNonAxi
     isList= isinstance(Pot,list)
-    if isList:
-        isAxis= [not p.isNonAxi for p in Pot]
-        nonAxi= not nu.prod(nu.array(isAxis))
-    else:
-        nonAxi= Pot.isNonAxi
+    nonAxi= _isNonAxi(Pot)
     if nonAxi and phi is None:
         raise PotentialError("The (list of) planarPotential instances is non-axisymmetric, but you did not provide phi")
     if isinstance(Pot,list) \
@@ -807,15 +1127,15 @@ def evaluateplanarR2derivs(R,Pot,phi=None,t=0.):
         sum= 0.
         for pot in Pot:
             if nonAxi:
-                sum+= pot.R2deriv(R,phi=phi,t=t)
+                sum+= pot.R2deriv(R,phi=phi,t=t,use_physical=False)
             else:
-                sum+= pot.R2deriv(R,t=t)
+                sum+= pot.R2deriv(R,t=t,use_physical=False)
         return sum
     elif isinstance(Pot,planarPotential):
         if nonAxi:
-            return Pot.R2deriv(R,phi=phi,t=t)
+            return Pot.R2deriv(R,phi=phi,t=t,use_physical=False)
         else:
-            return Pot.R2deriv(R,t=t)
+            return Pot.R2deriv(R,t=t,use_physical=False)
     else: #pragma: no cover 
         raise PotentialError("Input to 'evaluatePotentials' is neither a Potential-instance or a list of such instances")
 
@@ -834,9 +1154,9 @@ def LinShuReductionFactor(axiPot,R,sigmar,nonaxiPot=None,
 
        axiPot - The background, axisymmetric potential
 
-       R - Cylindrical radius
+       R - Cylindrical radius (can be Quantity)
        
-       sigmar - radial velocity dispersion of the population
+       sigmar - radial velocity dispersion of the population (can be Quantity)
 
        Then either provide:
 
@@ -844,7 +1164,7 @@ def LinShuReductionFactor(axiPot,R,sigmar,nonaxiPot=None,
 
           k= wavenumber (see Binney & Tremaine 2008)
 
-          OmegaP= pattern speed
+          OmegaP= pattern speed (can be Quantity)
 
        2) nonaxiPot= a non-axisymmetric Potential instance (such as SteadyLogSpiralPotential) that has functions that return OmegaP, m, and wavenumber
 
@@ -884,9 +1204,9 @@ def plotplanarPotentials(Pot,*args,**kwargs):
 
     INPUT:
 
-       Rrange - range
+       Rrange - range (can be Quantity)
 
-       xrange, yrange - if relevant
+       xrange, yrange - if relevant (can be Quantity)
 
        grid, gridx, gridy - number of points to plot
 
@@ -908,6 +1228,23 @@ def plotplanarPotentials(Pot,*args,**kwargs):
     Rrange= kwargs.pop('Rrange',[0.01,5.])
     xrange= kwargs.pop('xrange',[-5.,5.])
     yrange= kwargs.pop('yrange',[-5.,5.])
+    if _APY_LOADED:
+        if hasattr(Pot,'_ro'):
+            tro= Pot._ro
+        else:
+            tro= Pot[0]._ro
+        if isinstance(Rrange[0],units.Quantity):
+            Rrange[0]= Rrange[0].to(units.kpc).value/tro
+        if isinstance(Rrange[1],units.Quantity):
+            Rrange[1]= Rrange[1].to(units.kpc).value/tro
+        if isinstance(xrange[0],units.Quantity):
+            xrange[0]= xrange[0].to(units.kpc).value/tro
+        if isinstance(xrange[1],units.Quantity):
+            xrange[1]= xrange[1].to(units.kpc).value/tro
+        if isinstance(yrange[0],units.Quantity):
+            yrange[0]= yrange[0].to(units.kpc).value/tro
+        if isinstance(yrange[1],units.Quantity):
+            yrange[1]= yrange[1].to(units.kpc).value/tro
     grid= kwargs.pop('grid',100)
     gridx= kwargs.pop('gridx',100)
     gridy= kwargs.pop('gridy',gridx)
@@ -936,13 +1273,15 @@ def plotplanarPotentials(Pot,*args,**kwargs):
                         thisphi= nu.arcsin(ys[jj]/thisR)
                     else:
                         thisphi= -nu.arcsin(ys[jj]/thisR)+nu.pi
-                    potR[ii,jj]= evaluateplanarPotentials(thisR,Pot,
-                                                          phi=thisphi)
+                    potR[ii,jj]= evaluateplanarPotentials(Pot,thisR,
+                                                          phi=thisphi,
+                                                          use_physical=False)
         else:
             Rs= nu.linspace(Rrange[0],Rrange[1],grid)
             potR= nu.zeros(grid)
             for ii in range(grid):
-                potR[ii]= evaluateplanarPotentials(Rs[ii],Pot)
+                potR[ii]= evaluateplanarPotentials(Pot,Rs[ii],
+                                                   use_physical=False)
         if not savefilename is None:
             print("Writing planar savefile "+savefilename+" ...")
             savefile= open(savefilename,'wb')
