@@ -1,10 +1,12 @@
 from galpy.snapshot_src import Snapshot
 from galpy.snapshot_src import snapshotMovies
+import galpy.snapshot_src.nemo_util
 
 #
 # Functions
 #
 snapshotToMovie= snapshotMovies.snapshotToMovie
+nemo_util= galpy.snapshot_src.nemo_util
 
 #
 # Classes
