@@ -31,10 +31,19 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 #ifndef __BOVY_SYMPLECTICODE_H__
 #define __BOVY_SYMPLECTICODE_H__
+#ifdef __cplusplus
+extern "C" {
+#endif
+#include "signal.h"
 #include <galpy_potentials.h>
+/*
+  Global variables
+*/
+volatile sig_atomic_t interrupted;
 /*
   Function declarations
 */
+void handle_sigint(int);
 void leapfrog(void (*func)(double, double *, double *,
 			   int, struct potentialArg *),
 	      int,
@@ -74,4 +83,7 @@ double symplec6_estimate_step(void (*func)(double , double *, double *,int, stru
 			      double, double *,
 			      int,struct potentialArg *,
 			      double,double);
+#ifdef __cplusplus
+}
+#endif
 #endif /* bovy_symplecticode.h */
